# -*- coding: utf-8 -*-
import sys
import os
import json


from tornado.testing import AsyncHTTPTestCase
from tornado.escape import to_unicode, json_decode, json_encode, url_escape

from server import ReloadApplication

PROJECT_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
DATA_DIR = os.path.join(os.path.dirname(__file__), "data")

class TestCase(AsyncHTTPTestCase):

	def setUp(self):
		super(TestCase, self).setUp()
		self.initialize()

	def initialize(self):
		pass

	def get_app(self):
		self.server = ReloadApplication(port=8881)
		return self.server

class TestBase(TestCase):

	def initialize(self):
		sitePath = os.path.join(DATA_DIR, "TestSite")
		self.server.mount(sitePath)

	def testIndex(self):
		response = self.fetch("/TestSite/index.html", method="GET")
		self.assertEqual(response.code, 200, response.body)
		self.assertEqual(response.body, "Hello!\n<script src=\"/_server/server.js\"></script>")

	def testDefaultIndex(self):
		response = self.fetch("/TestSite/", method="GET")
		self.assertEqual(response.code, 200, response.body)
		self.assertEqual(response.body, "Hello!\n<script src=\"/_server/server.js\"></script>")

	def testNotAppend(self):
		response = self.fetch("/TestSite/test.js", method="GET")
		self.assertEqual(response.code, 200, response.body)
		self.assertEqual(response.body, "console.log(\"hello\")")

	def testServerScript(self):
		response = self.fetch("/_server/server.js", method="GET")
		
		with open(os.path.join(PROJECT_DIR, "www", "server.js"), "r") as f:
			self.assertEqual(response.code, 200, response.body)
			self.assertEqual(response.body, f.read())

class TestAPI(TestCase):

	def initialize(self):
		self.sitePath = os.path.join(DATA_DIR, "TestSite")
		self.server.mount(self.sitePath)

	def testList(self):
		response = self.fetch("/_server/api/list", method="GET")
		self.assertEqual(response.code, 200, response.body)
		body = json_decode(response.body)
		self.assertEqual(body["result"][0]["path"], self.sitePath, body)
		self.assertEqual(body["result"][0]["url"], "/TestSite/")

class TestAPIMount(TestCase):

	def initialize(self):
		self.sitePath = os.path.join(DATA_DIR, "TestSite")

	def testMount(self):
		response = self.fetch("/_server/api/mount", method="POST", body='{"path": "%s"}' % self.sitePath)
		self.assertEqual(response.code, 200, response.body)
		body = json_decode(response.body)
		self.assertEqual(body["result"]["path"], self.sitePath, body)

		response = self.fetch("/_server/api/list", method="GET")
		self.assertEqual(response.code, 200, response.body)
		body = json_decode(response.body)
		self.assertEqual(body["result"][0]["path"], self.sitePath, body)

	def testMountUnexisting(self):
		response = self.fetch("/_server/api/mount", method="POST", body='{"path": "/a/b/c"}')
		self.assertEqual(response.code, 500, response.body)
		self.assertEqual(response.body, '{"error": "Path does not exist /a/b/c"}')


class TestSecurity(TestCase):

	# def testInsecureMount(self):
	# 	for path in ["/etc", "/home", "/tmp"]:
	# 		response = self.fetch("/_server/api/mount", method="POST", body='{"path": "%s"}' % path)
	# 		self.assertEqual(response.code, 500)
	# 		body = json_decode(response.body)
	# 		self.assertEqual(body["error"], "reloader.mount: path not in user home path: {0}".format(path))

	def testPathTraversalServer(self):
		response = self.fetch("/_server/../Makefile", method="GET")
		self.assertEqual(response.code, 403)

	def testPathTraversalMount(self):

		path = os.path.join(DATA_DIR, "TestSite")
		response = self.fetch("/_server/api/mount", method="POST", body='{"path": "%s"}' % path)

		response = self.fetch("/TestSite/index.html", method="GET")
		self.assertEqual(response.code, 200, response.body)
		self.assertEqual(response.body, "Hello!\n<script src=\"/_server/server.js\"></script>")

		response = self.fetch("/TestSite/../test.py", method="GET")
		self.assertEqual(response.code, 403, response.body)

		response = self.fetch("/TestSite/..?///?...../test.py", method="GET")
		self.assertEqual(response.code, 403, response.body)


class TestAPIUnmount(TestCase):

	def initialize(self):
		self.sitePath = os.path.join(DATA_DIR, "TestSite")
		self.server.mount(self.sitePath)

	def testUnmount(self):
		response = self.fetch("/_server/api/unmount", method="POST", body='{"path": "%s"}' % self.sitePath)
		self.assertEqual(response.code, 200, response.body)
		self.assertEqual(response.body, '{"result": "OK"}')

		response = self.fetch("/_server/api/list", method="GET")
		self.assertEqual(response.code, 200, response.body)
		self.assertEqual(response.body, '{"result": []}')

class TestAPIEval(TestCase):

	def initialize(self):
		self.sitePath = os.path.join(DATA_DIR, "TestSite")
		self.server.mount(self.sitePath)

	def testEval(self):
		response = self.fetch("/_server/api/eval", method="POST", body=json_encode({"script": "console.log('hello')"}))
		self.assertEqual(response.code, 200, response.body)
		self.assertEqual(response.body, '{"result": "OK"}')

class TestFramerProject(TestCase):

	def initialize(self):
		self.sitePath = os.path.join(DATA_DIR, "TestFramerProject")
		self.fetch("/_server/api/mount", method="POST", body='{"path": "%s"}' % self.sitePath)

	def testList(self):
		response = self.fetch("/_server/api/list", method="GET")
		self.assertEqual(response.code, 200, response.body)
		body = json_decode(response.body)
		self.assertEqual(body["result"][0]["path"], self.sitePath, body)

	def testJSON(self):
		response = self.fetch("/TestFramerProject/imported/twitter-feed/layers.json", method="GET")
		self.assertEqual(response.code, 200, response.body)
		self.assertEqual(response.body, open(os.path.join(self.sitePath, "imported/twitter-feed/layers.json"), "r").read())

	def testImage(self):
		response = self.fetch("/TestFramerProject/imported/twitter-feed/images/Layer-avatar-1A13EAD8-4269-4694-87CC-3F66C9D1DC19.png", method="GET")
		self.assertEqual(response.code, 200, response.body)
		self.assertEqual(response.body, open(os.path.join(self.sitePath, "imported/twitter-feed/images/Layer-avatar-1A13EAD8-4269-4694-87CC-3F66C9D1DC19.png"), "r").read())

	# def testMountUnexisting(self):
	# 	response = self.fetch("/_server/api/mount", method="POST", body='{"path": "/a/b/c"}')
	# 	self.assertEqual(response.code, 500, response.body)
	# 	self.assertEqual(response.body, '{"error": "Path does not exist /a/b/c"}')

class TestUnicodeFramerProject(TestCase):

	def initialize(self):
		self.sitePath = os.path.join(DATA_DIR, u"TestFramerProjectβ")

		response = self.fetch("/_server/api/mount", method="POST", body=json.dumps({"path": self.sitePath}))
		body = json_decode(response.body)
		print response.body
		self.assertEqual(body["result"]["path"], self.sitePath)
		self.assertEqual(body["result"]["handle"], u"TestFramerProjectβ")

	def testList(self):
		response = self.fetch(u"/_server/api/list", method="GET")
		self.assertEqual(response.code, 200, response.body)
		body = json_decode(response.body)
		self.assertEqual(body["result"][0]["path"], self.sitePath, body)

	def testJSON(self):
		response = self.fetch(u"/{0}/imported/twitter-feed/layers.json".format(url_escape(u"TestFramerProjectβ")), method="GET")
		self.assertEqual(response.code, 200, response.body)
		self.assertEqual(response.body, open(os.path.join(self.sitePath, "imported/twitter-feed/layers.json"), "r").read())

	def testImage(self):
		response = self.fetch(u"/{0}/imported/twitter-feed/images/Layer-avatar-1A13EAD8-4269-4694-87CC-3F66C9D1DC19.png".format(url_escape(u"TestFramerProjectβ")), method="GET")
		self.assertEqual(response.code, 200, response.body)
		self.assertEqual(response.body, open(os.path.join(self.sitePath, "imported/twitter-feed/images/Layer-avatar-1A13EAD8-4269-4694-87CC-3F66C9D1DC19.png"), "r").read())




# class TestOfflineCache(TestCase):

# 	def initialize(self):
# 		self.sitePath = os.path.join(DATA_DIR, "TestSite")
# 		self.server.mount(self.sitePath)

# 	def testHtmlReplace(self):
# 		response = self.fetch("/TestSite/index.html", method="GET")
# 		self.assertEqual(response.code, 200, response.body)

# 		print response.body

# 	def testOfflineHtml(self):
# 		response = self.fetch("/_server/offline/TestSite/offline.html", method="GET")
# 		self.assertEqual(response.code, 200, response.body)

# 	def testOfflineCacheManifest(self):
# 		response = self.fetch("/_server/offline/TestSite/manifest.appcache", method="GET")
# 		self.assertEqual(response.code, 200, response.body)
# 		self.assertEqual(response.body, """CACHE MANIFEST
# /TestSite/index.html
# /TestSite/test.js""")

